package com.miya.miyalogger;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Environment;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;

public class MiyaLogger {

    private static File logFiles;//日志文件夹
    private static File logFile;//单个日志文件
    private static ScheduledExecutorService sExecutor;

    public static final String CACHE_FILE_PATH = Environment
            .getExternalStorageDirectory().getPath() +
            File.separator +
            "miyaterminal" +
            File.separator +
            "log" +
            File.separator;

    public static boolean isProhibitWrite = false;//是否禁止写入文件
    private static List<FileBean> allFileBeanList;//所有的文件列表
    private static List<File> deleteFilePathList;//需要删除的文件列表
    private static String logSuffix = ".txt";
    private static String logLink = "_";

    public static final String DF_YYYYMMDD = "yyyyMMdd";

    /**
     * 将日期转换成字符串
     *
     * @param date
     * @param format
     * @return
     */
    private static String format(Date date, String format) {
        if (date == null) {
            throw new IllegalArgumentException("Param date is null!");
        }
        if (TextUtils.isEmpty(format)) {
            throw new IllegalArgumentException("Param format is blank!");
        }
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

    private static String format8(Date date) {
        return format(date, DF_YYYYMMDD);
    }

    /**
     * 将不同格式的 String 转换成 时间戳
     *
     * @param str
     * @param formatStyle
     * @return
     */
    private static long stringToLong(String str, String formatStyle) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat(formatStyle);
        return sdf.parse(str).getTime();//毫秒
    }

    /**
     * 开启线程池，写日志，避免到处开启线程损耗性能
     *
     * @param input 写入内容
     * @return
     */
    public static boolean writeLog(final Context c, final String input) {
        if (sExecutor == null) {
            sExecutor = Executors.newScheduledThreadPool(5);
        }
        Future<Boolean> submit = sExecutor.submit(new Callable<Boolean>() {
            @Override
            public Boolean call() {
                try {
                    logFiles = new File(CACHE_FILE_PATH);
                    if (!logFiles.exists()) {
                        logFiles.mkdirs();
                    }

                    String logPath = CACHE_FILE_PATH + getVersionCode(c)
                            + "_" + format8(new Date()) + ".txt";

                    String message = input;

                    logFile = new File(logPath);
                    FileOutputStream fos = new FileOutputStream(logFile, true);
                    OutputStreamWriter writer = new OutputStreamWriter(fos, Charset.forName("utf-8"));
                    writer.write(message);
                    writer.write("\n");
                    writer.flush();
                    writer.close();
                    return true;
                } catch (IOException e) {
                    e.printStackTrace();
                    return false;
                }
            }
        });
        try {
            return submit.get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
        return false;
    }

    public static String getVersionCode(Context context) {
        PackageManager packageManager = context.getPackageManager();
        PackageInfo packageInfo;
        String versionCode = "";
        try {
            packageInfo = packageManager.getPackageInfo(context.getPackageName(), 0);
            versionCode = packageInfo.versionCode + "";
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionCode;
    }

    /**
     * 删除全部日志
     */
    public static void deleteAllLog() {
        File tempFile = new File(CACHE_FILE_PATH);
        if (tempFile == null || tempFile.listFiles() == null || tempFile.listFiles().length == 0) {
            return;
        }
        new DeleteLogTask().execute(tempFile.listFiles());
    }


    /**
     * 删除指定天数之前的日志，此处的天数不是精确到小时的，只精确到日
     *
     * @param days
     */
    public static void deleteLog(int days) {
        getFiles();
        if (deleteFilePathList == null) {
            deleteFilePathList = new ArrayList<>();
        } else {
            deleteFilePathList.clear();
        }
        if (allFileBeanList != null && allFileBeanList.size() > 0) {
            for (int i = 0; i < allFileBeanList.size(); i++) {
                getDeleteFileList(days, allFileBeanList.get(i));
            }
        }
        if (deleteFilePathList != null && deleteFilePathList.size() > 0) {
            File[] deleteFiles = deleteFilePathList.toArray(new File[0]);
            new DeleteLogTask().execute(deleteFiles);
        }
    }

    /**
     * 执行符合条件的删除日志操作
     *
     * @param days
     * @param fileBean
     */
    private static void getDeleteFileList(int days, FileBean fileBean) {
        long currentLongTimes = System.currentTimeMillis();
        if (fileBean.fileName.endsWith(logSuffix)) {
            if (fileBean.fileName.contains(logLink)) {
                try {
                    String fileNameWithOutSuffix = fileBean.fileName.substring(0, fileBean.fileName.lastIndexOf("."));
                    String fileNameDate = fileNameWithOutSuffix.substring(fileNameWithOutSuffix.lastIndexOf(logLink) + 1);
                    long fileNameLongTimes = stringToLong(fileNameDate, DF_YYYYMMDD);
                    if ((currentLongTimes - fileNameLongTimes) >= days * 24L * 60L * 60L * 1000L) {//符合要求，加入删除列表
                        deleteFilePathList.add(new File(fileBean.filePath));
                    }

                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 查询所有日志
     */
    public static List<String> getLogFileList() {
        File tempFile = new File(CACHE_FILE_PATH);
        if (tempFile != null) {
            List<String> result = new ArrayList<>();
            if (tempFile.listFiles() == null || tempFile.listFiles().length == 0) {
                return null;
            }
            for (File file : tempFile.listFiles()) {
                result.add(file.getName());
            }
            return result;
        }
        return null;
    }


    /**
     * 获得日志文件夹下的文件名和文件路径
     */
    private static void getFiles() {
        if (allFileBeanList == null) {
            allFileBeanList = new ArrayList<>();
        } else {
            allFileBeanList.clear();
        }
        File tempFile = new File(CACHE_FILE_PATH);
        if (tempFile == null) {
            return;
        }
        File[] files = tempFile.listFiles();
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                FileBean fileBean = new FileBean();
                fileBean.fileName = files[i].getName();
                fileBean.filePath = files[i].getAbsolutePath();
                allFileBeanList.add(fileBean);
            }
        }
    }


    private static class DeleteLogTask extends AsyncTask<File, Void, Boolean> {
        @Override
        protected Boolean doInBackground(File... files) {
            for (File file : files) {
                doDeleteFiles(file);
            }
            return true;
        }

        @Override
        protected void onPostExecute(Boolean aBoolean) {
            super.onPostExecute(aBoolean);
            if (aBoolean) {
                Log.i("LogFileUtils", "日志删除成功！");
            }
        }
    }

    private static void doDeleteFiles(File file) {
        if (file == null) {
            return;
        }
        //flie：要删除的文件夹的所在位置
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (int i = 0; i < files.length; i++) {
                File f = files[i];
                doDeleteFiles(f);
            }
            file.delete();//如要保留文件夹，只删除文件，请注释这行
        } else if (file.exists()) {
            file.delete();
        }
    }

    private static class FileBean {
        String fileName;
        String filePath;
    }

    //设为false关闭日志
    public static boolean LOG_ENABLE = true;

    public static void i(String tag, @NonNull String msg) {
        if (LOG_ENABLE) {
            //当文本量超过4K时，循环打印
            msg = msg.trim();
            int index = 0;
            int maxLength = 4000;
            String sub;
            while (index < msg.length()) {
                // java的字符不允许指定超过总的长度end
                if (msg.length() <= index + maxLength) {
                    sub = msg.substring(index);
                } else {
                    sub = msg.substring(index, index + maxLength);
                }
                index += maxLength;
                Log.i(tag, sub.trim());
            }
        }
    }

    public static void v(String tag, @NonNull String msg) {
        if (LOG_ENABLE) {
            //当文本量超过4K时，循环打印
            msg = msg.trim();
            int index = 0;
            int maxLength = 4000;
            String sub;
            while (index < msg.length()) {
                // java的字符不允许指定超过总的长度end
                if (msg.length() <= index + maxLength) {
                    sub = msg.substring(index);
                } else {
                    sub = msg.substring(index, index+maxLength);
                }
                index += maxLength;
                Log.v(tag, sub.trim());
            }
        }
    }

    public static void d(String tag, @NonNull String msg) {
        if (LOG_ENABLE) {
            //当文本量超过4K时，循环打印
            msg = msg.trim();
            int index = 0;
            int maxLength = 4000;
            String sub;
            while (index < msg.length()) {
                // java的字符不允许指定超过总的长度end
                if (msg.length() <= index + maxLength) {
                    sub = msg.substring(index);
                } else {
                    sub = msg.substring(index, index+maxLength);
                }
                index += maxLength;
                Log.d(tag, sub.trim());
            }
        }
    }

    public static void w(String tag, @NonNull String msg) {
        if (LOG_ENABLE) {
            //当文本量超过4K时，循环打印
            msg = msg.trim();
            int index = 0;
            int maxLength = 4000;
            String sub;
            while (index < msg.length()) {
                // java的字符不允许指定超过总的长度end
                if (msg.length() <= index + maxLength) {
                    sub = msg.substring(index);
                } else {
                    sub = msg.substring(index, index+maxLength);
                }
                index += maxLength;
                Log.w(tag, sub.trim());
            }
        }
    }

    public static void e(String tag, @NonNull String msg) {
        if (LOG_ENABLE) {
            //当文本量超过4K时，循环打印
            msg = msg.trim();
            int index = 0;
            int maxLength = 4000;
            String sub;
            while (index < msg.length()) {
                // java的字符不允许指定超过总的长度end
                if (msg.length() <= index + maxLength) {
                    sub = msg.substring(index);
                } else {
                    sub = msg.substring(index, index+maxLength);
                }
                index += maxLength;
                Log.e(tag, sub.trim());
            }
        }
    }
}
